package Bundle::DBD::InterBase;

$VERSION = '0.30';

1;

__END__

=head1 NAME

Bundle::DBD::InterBase - A bundle to install the DBD::InterBase driver

=head1 SYNOPSIS

C<perl -MCPAN -e 'install Bundle::DBD::InterBase'>

=head1 CONTENTS

DBI

DBD::InterBase

=head1 DESCRIPTION

This bundle includes all that's needed to run the DBI driver for InterBase.

=cut
